@@ -1,10 +1,7 @@
 import os
 import stat
-import re
-import six
 import tempfile
 import time
-import sys
 import shutil
 import json
 import uuid
@@ -104,6 +101,7 @@ def find_nvme_ns_devs_for_subnqn(subnqn):
 
     return ns_dev_paths
 
+
 def setup_nvme_target(dev_paths, subnqn):
     """
     Sets up a new NVMe target loop device (using nvmetcli) on top of the
@@ -174,6 +172,7 @@ def setup_nvme_target(dev_paths, subnqn):
 
 class UdisksNVMeTest(udiskstestcase.UdisksTestCase):
     SUBNQN = 'udisks_test_subnqn'
+    DISCOVERY_NQN = 'nqn.2014-08.org.nvmexpress.discovery'
     NUM_NS = 2
     NS_SIZE = 1024**3
 
@@ -186,21 +185,21 @@ class UdisksNVMeTest(udiskstestcase.UdisksTestCase):
         if not shutil.which("nvmetcli"):
             udiskstestcase.UdisksTestCase.tearDownClass()
             raise unittest.SkipTest("nvmetcli executable not found in $PATH, skipping.")
-        ret, out = udiskstestcase.run_command("modprobe nvme-fabrics")
+        ret, _out = udiskstestcase.run_command("modprobe nvme-fabrics")
         if ret != 0:
             raise unittest.SkipTest("nvme-fabrics kernel module unavailable, skipping.")
 
-        manager = cls.get_interface("/Manager", ".Manager")
-        cls.loop_devs = []
+        cls.dev_files = []
 
-        for i in range(cls.NUM_NS):
-            with tempfile.NamedTemporaryFile(prefix="udisks_test", delete=True, mode='w+b') as temp:
+        for _ in range(cls.NUM_NS):
+            with tempfile.NamedTemporaryFile(prefix="udisks_test", delete=False, mode='w+b', dir='/var/tmp') as temp:
                 temp.truncate(cls.NS_SIZE)
-                loop_dev_obj_path = manager.LoopSetup(temp.fileno(), cls.no_options)
-                path, loop_dev = loop_dev_obj_path.rsplit("/", 1)
-                cls.loop_devs += ["/dev/%s" % loop_dev]
+                cls.dev_files += [temp.name]
 
-        setup_nvme_target(cls.loop_devs, cls.SUBNQN)
+        setup_nvme_target(cls.dev_files, cls.SUBNQN)
+
+        for d in cls.dev_files:
+            os.unlink(d)
 
     def _nvme_disconnect(self, subnqn, ignore_errors=False):
         ret, out = self.run_command("nvme disconnect --nqn=%s" % subnqn)
@@ -224,12 +223,13 @@ class UdisksNVMeTest(udiskstestcase.UdisksTestCase):
         ret, out = udiskstestcase.run_command("nvmetcli clear")
         if ret != 0:
             raise RuntimeError("Error clearing the NVMe target: %s" % out)
-        # detach loop devices
-        for i in cls.loop_devs:
-            cls.run_command("losetup --detach %s" % i)
+        for d in cls.dev_files:
+            try:
+                os.unlink(d)
+            except FileNotFoundError:
+                pass
         udiskstestcase.UdisksTestCase.tearDownClass()
 
-
     def test_controller_info(self):
         self._nvme_connect()
         self.addCleanup(self._nvme_disconnect, self.SUBNQN, ignore_errors=True)
@@ -274,10 +274,9 @@ class UdisksNVMeTest(udiskstestcase.UdisksTestCase):
         fguid = self.get_property_raw(drive_obj, '.NVMe.Controller', 'FGUID')
         self.assertEqual(len(fguid), 0)
         rev = self.get_property_raw(drive_obj, '.NVMe.Controller', 'NVMeRevision')
-        self.assertTrue(rev.startswith('1.'))
+        self.assertGreater(len(rev), 0)
         unalloc_cap = self.get_property_raw(drive_obj, '.NVMe.Controller', 'UnallocatedCapacity')
-        self.assertEquals(unalloc_cap, 0)
-
+        self.assertEqual(unalloc_cap, 0)
 
     def test_namespace_info(self):
         self._nvme_connect()
@@ -305,16 +304,16 @@ class UdisksNVMeTest(udiskstestcase.UdisksTestCase):
             nguid = self.get_property_raw(ns, '.NVMe.Namespace', 'NGUID')
             self.assertGreater(len(nguid), 10)
             eui64 = self.get_property_raw(ns, '.NVMe.Namespace', 'EUI64')
-            self.assertEquals(len(eui64), 0)
+            self.assertEqual(len(eui64), 0)
             uuid = self.get_property_raw(ns, '.NVMe.Namespace', 'UUID')
             self.assertGreater(len(uuid), 10)
             wwn = self.get_property_raw(ns, '.NVMe.Namespace', 'WWN')
             self.assertTrue(wwn.startswith('uuid.'))
             lbaf = self.get_property_raw(ns, '.NVMe.Namespace', 'LBAFormats')
-            self.assertEquals(len(lbaf), 1)
-            self.assertEquals(lbaf[0], (512, 0, 1))
+            self.assertEqual(len(lbaf), 1)
+            self.assertEqual(lbaf[0], (4096, 0, 1))
             lbaf_curr = self.get_property_raw(ns, '.NVMe.Namespace', 'FormattedLBASize')
-            self.assertEquals(lbaf_curr, (512, 0, 1))
+            self.assertEqual(lbaf_curr, (4096, 0, 1))
             nsize = self.get_property_raw(ns, '.NVMe.Namespace', 'NamespaceSize')
             self.assertEqual(nsize, self.NS_SIZE / lbaf_curr[0])
             ncap = self.get_property_raw(ns, '.NVMe.Namespace', 'NamespaceCapacity')
@@ -324,7 +323,6 @@ class UdisksNVMeTest(udiskstestcase.UdisksTestCase):
             format_progress = self.get_property_raw(ns, '.NVMe.Namespace', 'FormatPercentRemaining')
             self.assertEqual(format_progress, -1)
 
-
     def test_health_info(self):
         self._nvme_connect()
         self.addCleanup(self._nvme_disconnect, self.SUBNQN, ignore_errors=True)
@@ -377,15 +375,14 @@ class UdisksNVMeTest(udiskstestcase.UdisksTestCase):
 
         # Try trigerring a self-test operation
         msg = 'The NVMe controller has no support for self-test operations'
-        with six.assertRaisesRegex(self, dbus.exceptions.DBusException, msg):
+        with self.assertRaisesRegex(dbus.exceptions.DBusException, msg):
             drive_obj.SmartSelftestStart('xxx', self.no_options, dbus_interface=self.iface_prefix + '.NVMe.Controller')
-        with six.assertRaisesRegex(self, dbus.exceptions.DBusException, msg):
+        with self.assertRaisesRegex(dbus.exceptions.DBusException, msg):
             drive_obj.SmartSelftestStart('short', self.no_options, dbus_interface=self.iface_prefix + '.NVMe.Controller')
         msg = 'NVMe Device Self-test command error: Invalid Command Opcode'
-        with six.assertRaisesRegex(self, dbus.exceptions.DBusException, msg):
+        with self.assertRaisesRegex(dbus.exceptions.DBusException, msg):
             drive_obj.SmartSelftestAbort(self.no_options, dbus_interface=self.iface_prefix + '.NVMe.Controller')
 
-
     def test_sanitize(self):
         self._nvme_connect()
         self.addCleanup(self._nvme_disconnect, self.SUBNQN, ignore_errors=True)
@@ -412,17 +409,16 @@ class UdisksNVMeTest(udiskstestcase.UdisksTestCase):
 
         # Try trigerring a sanitize operation
         msg = 'Unknown sanitize action xxx'
-        with six.assertRaisesRegex(self, dbus.exceptions.DBusException, msg):
+        with self.assertRaisesRegex(dbus.exceptions.DBusException, msg):
             drive_obj.SanitizeStart('xxx', self.no_options, dbus_interface=self.iface_prefix + '.NVMe.Controller')
         msg = r'The NVMe controller has no support for the .* sanitize operation'
-        with six.assertRaisesRegex(self, dbus.exceptions.DBusException, msg):
+        with self.assertRaisesRegex(dbus.exceptions.DBusException, msg):
             drive_obj.SanitizeStart('block-erase', self.no_options, dbus_interface=self.iface_prefix + '.NVMe.Controller')
-        with six.assertRaisesRegex(self, dbus.exceptions.DBusException, msg):
+        with self.assertRaisesRegex(dbus.exceptions.DBusException, msg):
             drive_obj.SanitizeStart('crypto-erase', self.no_options, dbus_interface=self.iface_prefix + '.NVMe.Controller')
-        with six.assertRaisesRegex(self, dbus.exceptions.DBusException, msg):
+        with self.assertRaisesRegex(dbus.exceptions.DBusException, msg):
             drive_obj.SanitizeStart('overwrite', self.no_options, dbus_interface=self.iface_prefix + '.NVMe.Controller')
 
-
     def test_format_ns(self):
         self._nvme_connect()
         self.addCleanup(self._nvme_disconnect, self.SUBNQN, ignore_errors=True)
@@ -442,50 +438,50 @@ class UdisksNVMeTest(udiskstestcase.UdisksTestCase):
             state.assertEqual('live', timeout=10)
 
             msg = 'Format NVM command error: Invalid Command Opcode: A reserved coded value or an unsupported value in the command opcode field'
-            with six.assertRaisesRegex(self, dbus.exceptions.DBusException, msg):
+            with self.assertRaisesRegex(dbus.exceptions.DBusException, msg):
                 ns.FormatNamespace(self.no_options, dbus_interface=self.iface_prefix + '.NVMe.Namespace')
 
             d = dbus.Dictionary(signature='sv')
-            with six.assertRaisesRegex(self, dbus.exceptions.DBusException, 'Unknown secure erase type xxx'):
+            with self.assertRaisesRegex(dbus.exceptions.DBusException, 'Unknown secure erase type xxx'):
                 d['secure_erase'] = 'xxx'
                 ns.FormatNamespace(d, dbus_interface=self.iface_prefix + '.NVMe.Namespace')
-            with six.assertRaisesRegex(self, dbus.exceptions.DBusException, msg):
+            with self.assertRaisesRegex(dbus.exceptions.DBusException, msg):
                 d['secure_erase'] = 'user_data'
                 ns.FormatNamespace(d, dbus_interface=self.iface_prefix + '.NVMe.Namespace')
-            with six.assertRaisesRegex(self, dbus.exceptions.DBusException, msg):
+            with self.assertRaisesRegex(dbus.exceptions.DBusException, msg):
                 d['secure_erase'] = 'crypto_erase'
                 ns.FormatNamespace(d, dbus_interface=self.iface_prefix + '.NVMe.Namespace')
 
             d = dbus.Dictionary(signature='sv')
-            with six.assertRaisesRegex(self, dbus.exceptions.DBusException, msg):
+            with self.assertRaisesRegex(dbus.exceptions.DBusException, msg):
                 d['lba_data_size'] = 0
                 ns.FormatNamespace(d, dbus_interface=self.iface_prefix + '.NVMe.Namespace')
             lbaf_curr = self.get_property_raw(ns, '.NVMe.Namespace', 'FormattedLBASize')
-            with six.assertRaisesRegex(self, dbus.exceptions.DBusException, msg):
+            with self.assertRaisesRegex(dbus.exceptions.DBusException, msg):
                 d['lba_data_size'] = lbaf_curr[0]
                 ns.FormatNamespace(d, dbus_interface=self.iface_prefix + '.NVMe.Namespace')
-            with six.assertRaisesRegex(self, dbus.exceptions.DBusException, msg):
+            with self.assertRaisesRegex(dbus.exceptions.DBusException, msg):
                 d['lba_data_size'] = lbaf_curr[0]
                 d['metadata_size'] = lbaf_curr[1]
                 ns.FormatNamespace(d, dbus_interface=self.iface_prefix + '.NVMe.Namespace')
 
             msg = "Couldn't match desired LBA data block size in a device supported LBA format data sizes"
-            with six.assertRaisesRegex(self, dbus.exceptions.DBusException, msg):
+            with self.assertRaisesRegex(dbus.exceptions.DBusException, msg):
                 d['lba_data_size'] = dbus.UInt16(666)
                 ns.FormatNamespace(d, dbus_interface=self.iface_prefix + '.NVMe.Namespace')
-            with six.assertRaisesRegex(self, dbus.exceptions.DBusException, msg):
+            with self.assertRaisesRegex(dbus.exceptions.DBusException, msg):
                 d['lba_data_size'] = lbaf_curr[0]
                 d['metadata_size'] = dbus.UInt16(5)
                 ns.FormatNamespace(d, dbus_interface=self.iface_prefix + '.NVMe.Namespace')
 
     def test_fabrics_connect(self):
         manager = self.get_interface("/Manager", ".Manager.NVMe")
-        with six.assertRaisesRegex(self, dbus.exceptions.DBusException, 'Invalid value specified for the transport address argument'):
+        with self.assertRaisesRegex(dbus.exceptions.DBusException, 'Invalid value specified for the transport address argument'):
             manager.Connect(self.str_to_ay(self.SUBNQN), "notransport", "", self.no_options)
         msg = r'Error connecting the controller: failed to write to nvme-fabrics device'
-        with six.assertRaisesRegex(self, dbus.exceptions.DBusException, msg):
+        with self.assertRaisesRegex(dbus.exceptions.DBusException, msg):
             manager.Connect(self.str_to_ay(self.SUBNQN), "loop", "127.0.0.1", self.no_options)
-        with six.assertRaisesRegex(self, dbus.exceptions.DBusException, msg):
+        with self.assertRaisesRegex(dbus.exceptions.DBusException, msg):
             manager.Connect(self.str_to_ay("unknownsubnqn"), "loop", "", self.no_options)
 
         d = dbus.Dictionary(signature='sv')
@@ -509,28 +505,89 @@ class UdisksNVMeTest(udiskstestcase.UdisksTestCase):
 
         ctrl.Disconnect(self.no_options, dbus_interface=self.iface_prefix + '.NVMe.Fabrics')
         ctrl = self.get_object(ctrl_obj_path)
-        with six.assertRaisesRegex(self, dbus.exceptions.DBusException, r'Object does not exist at path .*|No such interface'):
+        with self.assertRaisesRegex(dbus.exceptions.DBusException, r'Object does not exist at path .*|No such interface'):
             self.get_property_raw(ctrl, '.NVMe.Fabrics', 'HostNQN')
 
+    def test_persistent_dc(self):
+        manager = self.get_interface("/Manager", ".Manager.NVMe")
+        with self.assertRaisesRegex(dbus.exceptions.DBusException, 'Invalid value specified for the transport address argument'):
+            manager.Connect(self.str_to_ay(self.DISCOVERY_NQN), "notransport", "", self.no_options)
+        msg = r'Error connecting the controller: failed to write to nvme-fabrics device'
+        with self.assertRaisesRegex(dbus.exceptions.DBusException, msg):
+            manager.Connect(self.str_to_ay(self.DISCOVERY_NQN), "loop", "127.0.0.1", self.no_options)
+
+        ctrl_obj_path = manager.Connect(self.str_to_ay(self.DISCOVERY_NQN), "loop", "", self.no_options)
+        self.addCleanup(self._nvme_disconnect, self.DISCOVERY_NQN, ignore_errors=True)
+
+        ctrl = self.get_object(ctrl_obj_path)
+        self.assertHasIface(ctrl, 'org.freedesktop.UDisks2.NVMe.Controller')
+        self.assertHasIface(ctrl, 'org.freedesktop.UDisks2.NVMe.Fabrics')
+
+        # health information should never be updated on a discovery controller
+        smart_timestamp = self.get_property_raw(ctrl, '.NVMe.Controller', 'SmartUpdated')
+        self.assertEqual(smart_timestamp, 0)
+        nqn = self.get_property_raw(ctrl, '.NVMe.Controller', 'SubsystemNQN')
+        self.assertEqual(nqn, self.str_to_ay(self.DISCOVERY_NQN))
+        transport = self.get_property_raw(ctrl, '.NVMe.Fabrics', 'Transport')
+        self.assertEqual(transport, 'loop')
+        tr_addr = self.get_property_raw(ctrl, '.NVMe.Fabrics', 'TransportAddress')
+        self.assertEqual(len(tr_addr), 1)   # the zero trailing byte
+
+        # make sure that no block device object references this drive object
+        udisks = self.get_object('')
+        objects = udisks.GetManagedObjects(dbus_interface='org.freedesktop.DBus.ObjectManager')
+        block_paths = [p for p in list(objects.keys()) if '/block_devices/' in p]
+        for p in block_paths:
+            self.assertNotEqual(str(objects[p]['org.freedesktop.UDisks2.Block']['Drive']), str(ctrl_obj_path))
+
+        # the following methods should fail on discovery controllers
+        msg = 'NVMe Health Information is only supported on I/O controllers'
+        with self.assertRaisesRegex(dbus.exceptions.DBusException, msg):
+            ctrl.SmartUpdate(self.no_options, dbus_interface=self.iface_prefix + '.NVMe.Controller')
+        msg = 'SMART data not collected'
+        with self.assertRaisesRegex(dbus.exceptions.DBusException, msg):
+            ctrl.SmartGetAttributes(self.no_options, dbus_interface=self.iface_prefix + '.NVMe.Controller')
+
+        msg = 'The NVMe controller has no support for self-test operations'
+        with self.assertRaisesRegex(dbus.exceptions.DBusException, msg):
+            ctrl.SmartSelftestStart('xxx', self.no_options, dbus_interface=self.iface_prefix + '.NVMe.Controller')
+        with self.assertRaisesRegex(dbus.exceptions.DBusException, msg):
+            ctrl.SmartSelftestStart('short', self.no_options, dbus_interface=self.iface_prefix + '.NVMe.Controller')
+        msg = 'NVMe Device Self-test command error: Invalid Command Opcode'
+        with self.assertRaisesRegex(dbus.exceptions.DBusException, msg):
+            ctrl.SmartSelftestAbort(self.no_options, dbus_interface=self.iface_prefix + '.NVMe.Controller')
+
+        msg = r'The NVMe controller has no support for the .* sanitize operation'
+        with self.assertRaisesRegex(dbus.exceptions.DBusException, msg):
+            ctrl.SanitizeStart('block-erase', self.no_options, dbus_interface=self.iface_prefix + '.NVMe.Controller')
+        with self.assertRaisesRegex(dbus.exceptions.DBusException, msg):
+            ctrl.SanitizeStart('crypto-erase', self.no_options, dbus_interface=self.iface_prefix + '.NVMe.Controller')
+        with self.assertRaisesRegex(dbus.exceptions.DBusException, msg):
+            ctrl.SanitizeStart('overwrite', self.no_options, dbus_interface=self.iface_prefix + '.NVMe.Controller')
+
+        ctrl.Disconnect(self.no_options, dbus_interface=self.iface_prefix + '.NVMe.Fabrics')
+        ctrl = self.get_object(ctrl_obj_path)
+        with self.assertRaisesRegex(dbus.exceptions.DBusException, r'Object does not exist at path .*|No such interface'):
+            self.get_property_raw(ctrl, '.NVMe.Fabrics', 'HostNQN')
 
     def test_hostnqn(self):
         HOSTNQN_PATH = '/etc/nvme/hostnqn'
         HOSTID_PATH = '/etc/nvme/hostid'
         FAKE_HOSTNQN = 'nqn.2014-08.org.nvmexpress:uuid:beefbeef-beef-beef-beef-beefdeadbeef'
-        FAKE_HOSTID = 'beefbeef-beef-beef-beef-beefdeadbeef'
+        FAKE_HOSTID = 'beeeeeef-beef-beef-beef-beefdeadbeef'
 
         if PACKAGE_SYSCONF_DIR != '/etc':
             self.skipTest("UDisks has been configured in non-system prefix, skipping...")
 
         # save hostnqn and hostid files
         try:
-            saved_hostnqn = read_file(HOSTNQN_PATH)
-            self.addCleanup(write_file, HOSTNQN_PATH, saved_hostnqn)
+            saved_hostnqn = self.read_file(HOSTNQN_PATH)
+            self.addCleanup(self.write_file, HOSTNQN_PATH, saved_hostnqn)
         except:
             self.addCleanup(self.remove_file, HOSTNQN_PATH, ignore_nonexistent=True)
         try:
-            saved_hostid = read_file(HOSTID_PATH)
-            self.addCleanup(write_file, HOSTID_PATH, saved_hostid)
+            saved_hostid = self.read_file(HOSTID_PATH)
+            self.addCleanup(self.write_file, HOSTID_PATH, saved_hostid)
         except:
             self.addCleanup(self.remove_file, HOSTID_PATH, ignore_nonexistent=True)
         self.remove_file(HOSTNQN_PATH, ignore_nonexistent=True)
