@@ -234,15 +234,6 @@ strip_and_replace_with_uscore (gchar *s)
   ;
 }
 
-static gboolean
-is_dm_multipath (UDisksLinuxDevice *device)
-{
-  const gchar *dm_uuid;
-
-  dm_uuid = g_udev_device_get_sysfs_attr (device->udev_device, "dm/uuid");
-  return dm_uuid != NULL && g_str_has_prefix (dm_uuid, "mpath-");
-}
-
 static void
 udisks_linux_drive_object_constructed (GObject *_object)
 {
@@ -437,7 +428,7 @@ udisks_linux_drive_object_get_device (UDisksLinuxDriveObject *object,
   g_mutex_lock (&object->devices_mutex);
   for (devices = object->devices; devices; devices = devices->next)
     {
-      if (!get_hw || !is_dm_multipath (UDISKS_LINUX_DEVICE (devices->data)))
+      if (!get_hw || !udisks_linux_device_is_dm_multipath (UDISKS_LINUX_DEVICE (devices->data)))
         {
           ret = devices->data;
           break;
@@ -486,7 +477,7 @@ udisks_linux_drive_object_get_block (UDisksLinuxDriveObject *object,
 
       device = udisks_linux_block_object_get_device (UDISKS_LINUX_BLOCK_OBJECT (iter_object));
       skip = (g_strcmp0 (g_udev_device_get_devtype (device->udev_device), "disk") != 0
-              || (get_hw && is_dm_multipath (device)));
+              || (get_hw && udisks_linux_device_is_dm_multipath (device)));
       g_object_unref (device);
 
       if (skip)
@@ -614,7 +605,8 @@ drive_ata_check (UDisksObject *object)
     goto out;
 
   device = drive_object->devices->data;
-  if (device->ata_identify_device_data != NULL || device->ata_identify_packet_device_data != NULL)
+  if (g_udev_device_get_property_as_boolean (device->udev_device, "ID_ATA") ||
+      device->ata_identify_device_data != NULL || device->ata_identify_packet_device_data != NULL)
     ret = TRUE;
 
  out:
@@ -1049,7 +1041,7 @@ udisks_linux_drive_object_should_include_device (GUdevClient        *client,
         }
 
       /* dm-multipath */
-      if (is_dm_multipath (device))
+      if (udisks_linux_device_is_dm_multipath (device))
         {
           gchar **slaves;
           guint n;
@@ -1116,9 +1108,8 @@ udisks_linux_drive_object_housekeeping (UDisksLinuxDriveObject  *object,
                                         GCancellable            *cancellable,
                                         GError                 **error)
 {
-  gboolean ret;
-
-  ret = FALSE;
+  UDisksLinuxDevice *device = NULL;
+  gboolean ret = FALSE;
 
   /* ATA */
   if (object->iface_drive_ata != NULL &&
@@ -1144,47 +1135,62 @@ udisks_linux_drive_object_housekeeping (UDisksLinuxDriveObject  *object,
                                                       cancellable,
                                                       &local_error))
         {
-          if (nowakeup && (local_error->domain == UDISKS_ERROR &&
-                           local_error->code == UDISKS_ERROR_WOULD_WAKEUP))
+          if (nowakeup && g_error_matches (local_error, UDISKS_ERROR, UDISKS_ERROR_WOULD_WAKEUP))
             {
               udisks_info ("Drive %s is in a sleep state",
                            g_dbus_object_get_object_path (G_DBUS_OBJECT (object)));
               g_clear_error (&local_error);
             }
-          else if (nowakeup && (local_error->domain == UDISKS_ERROR &&
-                                local_error->code == UDISKS_ERROR_DEVICE_BUSY))
+          else if (nowakeup && g_error_matches (local_error, UDISKS_ERROR, UDISKS_ERROR_DEVICE_BUSY))
             {
               /* typically because a "secure erase" operation is pending */
               udisks_info ("Drive %s is busy",
                            g_dbus_object_get_object_path (G_DBUS_OBJECT (object)));
               g_clear_error (&local_error);
             }
+          else if (g_error_matches (local_error, UDISKS_ERROR, UDISKS_ERROR_CANCELLED))
+            {
+              /* typically because the device indicates it refuses any I/O intentionally */
+              udisks_info ("Drive %s is refusing any I/O intentionally",
+                           g_dbus_object_get_object_path (G_DBUS_OBJECT (object)));
+              g_clear_error (&local_error);
+            }
           else
             {
+              /* all other errors are reported within the BD_SMART_ERROR domain */
               g_propagate_prefixed_error (error, local_error, "Error updating SMART data: ");
               goto out;
             }
         }
     }
   /* NVMe */
-  if (object->iface_nvme_ctrl != NULL)
+  if (object->iface_nvme_ctrl != NULL &&
+      g_strcmp0 (udisks_nvme_controller_get_state (UDISKS_NVME_CONTROLLER (object->iface_nvme_ctrl)), "live") == 0)
     {
       GError *local_error = NULL;
 
-      udisks_info ("Refreshing Health Information on %s",
-                   g_dbus_object_get_object_path (G_DBUS_OBJECT (object)));
-
-      if (!udisks_linux_nvme_controller_refresh_smart_sync (UDISKS_LINUX_NVME_CONTROLLER (object->iface_nvme_ctrl),
-                                                            cancellable, &local_error))
+      /* Only perform health check on I/O controllers */
+      device = udisks_linux_drive_object_get_device (object, TRUE /* get_hw */);
+      if (device && device->nvme_ctrl_info &&
+          (device->nvme_ctrl_info->controller_type == BD_NVME_CTRL_TYPE_IO ||
+           device->nvme_ctrl_info->controller_type == BD_NVME_CTRL_TYPE_UNKNOWN))
         {
-          g_propagate_prefixed_error (error, local_error, "Error updating Health Information: ");
-          goto out;
+          udisks_info ("Refreshing Health Information on %s",
+                       g_dbus_object_get_object_path (G_DBUS_OBJECT (object)));
+
+          if (!udisks_linux_nvme_controller_refresh_smart_sync (UDISKS_LINUX_NVME_CONTROLLER (object->iface_nvme_ctrl),
+                                                                cancellable, &local_error))
+            {
+              g_propagate_prefixed_error (error, local_error, "Error updating Health Information: ");
+              goto out;
+            }
         }
     }
 
   ret = TRUE;
 
  out:
+  g_clear_object (&device);
   return ret;
 }
 
